<?php
/**
 * @package     CSVI
 * @subpackage  CSVI
 *
 * @author      RolandD Cyber Produksi <contact@csvimproved.com>
 * @copyright   Copyright (C) 2006 - @year@ RolandD Cyber Produksi. All rights reserved.
 * @license     GNU/GPLv3 http://www.gnu.org/licenses/gpl-3.0.html
 * @link        https://csvimproved.com
 */

namespace csvi\com_csvi\model\import;

defined('_JEXEC') or die;

/**
 * Custom import.
 *
 * @package     CSVI
 * @subpackage  CSVI
 * @since       6.0
 */
class Custom extends \RantaiImportEngine
{
	/**
	 * Custom table
	 *
	 * @var    \CsviTableCustomtable
	 * @since  6.0
	 */
	private $customTable = null;

	/**
	 * The primary key field
	 *
	 * @var    mixed
	 * @since  6.0
	 */
	private $pk = null;

	/**
	 * Start the product import process.
	 *
	 * @return  bool  True on success | false on failure.
	 *
	 * @since   6.0
	 */
	public function getStart()
	{
		// Process data
		foreach ($this->fields->getData() as $fields)
		{
			foreach ($fields as $name => $details)
			{
				$value = $details->value;

				switch ($name)
				{
					default:
						$this->setState($name, $value);
						break;
				}
			}
		}

		// Find the primary key
		$this->pk = $this->customTable->getKeyName(true);

		if (!$this->pk)
		{
			$this->log->addStats('skipped', \JText::_('COM_CSVI_CUSTOM_NO_PRIMARY_KEY_AND_NO_FIELDS_SET'));

			return false;
		}

		$this->loaded = true;
		$this->log->add('Processed record based on fields: ' . implode(',', $this->pk), false);
		$this->customTable->bind($this->state);

		if ($this->customTable->checkIfRowExists())
		{
			if (!$this->template->get('overwrite_existing_data'))
			{
				$this->log->add(\JText::sprintf('COM_CSVI_DATA_EXISTS_CUSTOM', implode(',', $this->pk)));
				$this->log->addStats('skipped', \JText::sprintf('COM_CSVI_DATA_EXISTS_CUSTOM', implode(',', $this->pk)));
				$this->loaded = false;
			}
			else
			{
				foreach ($this->pk as $key => $value)
				{
					if ($currentValue = $this->customTable->get($value, 0))
					{
						$this->setState($value, $currentValue);
					}
				}
			}
		}

		return true;
	}

	/**
	 * Process a record.
	 *
	 * @return  bool  Returns true if all is OK | Returns false if no product SKU or product ID can be found.
	 *
	 * @since   6.0
	 */
	public function getProcessRecord()
	{
		if (!$this->loaded)
		{
			return false;
		}

		$rowExists = $this->customTable->checkIfRowExists();

		if (!$rowExists && $this->template->get('ignore_non_exist'))
		{
			// Do nothing for new records when user chooses to ignore new products
			$this->log->addStats('skipped', \JText::sprintf('COM_CSVI_DATA_EXISTS_IGNORE_NEW', $this->getState($this->pk, '')));

			return false;
		}

		try
		{
			if ($rowExists)
			{
				$this->db->updateObject($this->customTable->getTableName(), $this->state, $this->pk);
			}
			else
			{
				$this->db->insertObject($this->customTable->getTableName(), $this->state);
			}

			$this->customTable->reset();
		}
		catch (\Exception $e)
		{
			$this->log->add('Cannot add custom table data. Error: ' . $e->getMessage(), false);
			$this->log->addStats('incorrect', $e->getMessage());

			return false;
		}

		return true;
	}

	/**
	 * Load the necessary tables.
	 *
	 * @return  void.
	 *
	 * @since   6.0
	 */
	public function loadTables()
	{
		$this->customTable = $this->getTable('CustomTable');
	}

	/**
	 * Clear the loaded tables.
	 *
	 * @return  void.
	 *
	 * @since   6.0
	 */
	public function clearTables()
	{
		$this->customTable->reset();
	}
}
